#include "config.h"
#include <OctoPrintAPI.h>
#include <WiFiClient.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include "tunes.h"

void playTune(int tuneNumber);
WiFiClient client;

OctoprintApi api(client, OCTOPRINT_HOST, OCTOPRINT_PORT, OCTOPRINT_API_KEY);
unsigned long api_mtbs = OCTOPRINT_API_REFRESH_INTERVAL;
unsigned long api_lasttime = 0;

bool displayOctoPrintVersion(Adafruit_SSD1306& display) {
    display.clearDisplay();
    display.setCursor(0, 0);
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.print("Fetching OctoPrint Version...");
    display.display();

    if (api.getOctoprintVersion()) {
        display.clearDisplay();
        display.setCursor(0, 0);
        display.setTextSize(1);
        display.setTextColor(WHITE);
        display.print("OctoPrint Version:");
        display.setCursor(0, 10);
        display.setTextSize(2);
        display.print(api.octoprintVer.octoprintServer);
        display.display();
        playTune(2);
        return true;
    } else {
        display.clearDisplay();
        display.setCursor(0, 0);
        display.setTextSize(1);
        display.setTextColor(WHITE);
        display.print("Failed to get");
        display.setCursor(0, 10);
        display.print("OctoPrint version.");
        display.display();
        playTune(3);
        return false;
    }
    delay(1000);
}

bool fetchPrintingStatus(Adafruit_SSD1306& display) {
    if (millis() - api_lasttime > api_mtbs || api_lasttime == 0) {
        if (api.getPrinterStatistics()) {
            display.clearDisplay();
            display.setCursor(0, 0);
            display.setTextSize(1);
            display.setTextColor(WHITE);
            display.println(api.printerStats.printerState);
            display.print("Nozzle: ");
            display.println(api.printerStats.printerTool0TempActual);
            display.print("Bed: ");
            display.println(api.printerStats.printerBedTempActual);

            //Print time left (if printing) in human readable time HH:MM:SS
            int runHours= api.printJob.progressPrintTimeLeft/3600;
            int secsRemaining=api.printJob.progressPrintTimeLeft%3600;
            int runMinutes=secsRemaining/60;
            int runSeconds=secsRemaining%60;
            char buf[31];
            sprintf(buf,"%02d:%02d:%02d",runHours,runMinutes,runSeconds);
            display.println(buf);

            const float temp_percent = floor(api.printJob.progressCompletion*100)/100;
            display.print(temp_percent);
            display.println("%");
            display.display();
            api_lasttime = millis(); // Update the last fetch time
            return true;
        } else {
            Serial.println("Failed to fetch printer statistics.");
            return false;
        }
    }
    return false; // No fetch was done (within the time interval)
}
